% Autor: Wiebke Petersen
% Sudoku Test Suite.: Allows to test and compare different sudoku implementations.
% Datum: 27.04.2017
%
% only use it, if you have a better solution for play/2 or a lot of time.

:- use_module(library(lists)).
% sudoku_boards.pl must provide a predicate game/2 representing sudoku boards
% and a predicate print_board/1 printing a board to the screen.
% sudoku_solver.pl  must provide a predicate play/2 that solves a sudoku board.
% Note that the board representation format in sudoku_boards.pl must fit the needs of your play/2
% predicate in sudoku_solver.pl


%:- consult(sudoku_boards_clp).
%:- consult(sudoku_solver_clp).

:- consult(sudoku_boards).
:- consult(sudoku_solver_basis).

% Write your own script sudoku_solver_improved_yourname.pl containing your improved solver.
%:- consult(sudoku_boards).
%:- consult(sudoku_solver_improved_yourname).

solve_all:-
        abolish(info:num/2),
        game(N,_),
        all_solutions(N),
        fail.

solve_all.


all_solutions(N):-
        statistics(walltime,_),
        abolish(info:game/2),!,
        solve_game(N),
        statistics(walltime,[_,T]),
        findall(N,clause(info:game(N,_),true),NL),
        length(NL,Z),
        write(spiel),tab(1),write(N),tab(4),write(time:T),nl,   % T is the time difference in msec
        write('Zahl der Lsungen:'),tab(2),write(Z),nl,
        ZD is T / Z,
        write('Durchschnittszeit:'),tab(2),write(ZD),nl,nl,
        assert(info:num(N,Z)).


solve_game(N):-
        game(N,G),
        play(G,LG),
        assert_game(N,LG),
        fail.

solve_game(_).


% assert_game/2: schreibt die Lsung in die Datenbasis
assert_game(N,NG):-
        sort(NG,NGS),
        assert(info:game(N,NGS)),!.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Ausgabeprdikate

% print_all/1: schreibt alle Lsungen eines Spiels auf den Bildschirm
% the predicate print_board/1 must be defined in sudoku_solver.

print_all(N):-
        write("game"),write(N),
        game(N,G),
        print_board(G),
        info:game(N,NG),
        write("solution for game"),write(N),
        print_board(NG),
        fail.

print_all(_).


